% Script to extract the chlorophyll a fluorescence signal from red-green-blue images. 
% The chlorophyll a fluorescence signal is given as intensity of the red channel based 
% on red-green-blue images. Process is described in Friedrichs, A, J.A. Busch, H.J. van der Woerd
% and O. Zielinski, SmartFluo: A Method and Affordable Adapter to Measure Chlorophyll a Fluorescence 
% with Smartphones, Sensors, 2017, 17, 678; doi:10.3390/s17040678.
%
% This script imports JPEG formatted images given by a camera. In this
% example JPEG images are taken by S3 mini smartphone camera with 2560x1920
% pixels. It is required that the user has sample images as well as at
% least one reference image stored in two subfolders, here called "Blank"
% and "Samples". User can rename them and change the values in the
% "Parameters" section below. Relative paths are used.
% 
% Furthermore, script imports the illumination time as a setting from the
% digital camera system. For the S3 mini smartphone this is called by
% <variablename>.DigitalCamera.ExposureTime. This can vary for different
% camera systems. If required changes can be done in sub-function for Step
% 3 (illuminationtimeextract).
% 
% All images are treated separately (even if duplicates or triplicates or 
% even more samples per concentration are available). In sub-function for
% Step 5 (blankcorrection) all processed reference images are averaged to 
% one mean reference to be used for base-line correction. At the end of the
% script the base-line corrected intensity is average over x and y axes
% (along pixel dimension) achieving one intensity value per image.
%  
% Finally results are given in AveragedIntensityPerSubImage.
% 
% The script is composed of five sub-functions called in five steps
% representing five required steps to achieve intensity of chlorophyll a
% fluorescence signal of RGB-images. 
%
% Contact details:
% Anna Friedrichs
% contact: anna.friedrichs@uol.de
% 
% Prof. Dr. Oliver Zielinski
% Institute for Chemistry and Biology of the Marine Environment
% University of Oldenburg
% Contact: oliver.zielinski@uol.de
% 
% Changelog:
%
% Version 1.0: Finished (June 2016)
% Version 1.1: Combination of all sub-functions in one script for distribution (02.04.2020)
%
% License information:
% Copyright (c) [2020] [Anna Friedrichs]
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included 
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
% MERCHANTABILITY,FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
% IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
% CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
% OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR 
% THE USE OR OTHER DEALINGS IN THE SOFTWARE.
% -----------------------------------

clearvars; close all; clc;
disp('RGB2Chl algorithm')
%% PARAMETERS %%%
% Define number of pixels. The double is used as sub-image size
SubImageSize=150;
% Allocate folders where reference images and sample images are stored
ReferenceFolder='Blank';
SampleFolder='Samples';
% Define image format: Script was developed with .jpg images
ImageFormat = '*.jpg';

%% INFORMATION
% List images in specified folders with specified format
ReferenceImages=dir([ReferenceFolder,'/',ImageFormat]);
SampleImages=dir([SampleFolder,'/',ImageFormat]);

disp(['*** There are ',num2str(length(SampleImages)),' RGB images'])
disp(['*** There are ',num2str(length(ReferenceImages)),' reference images'])

%% IMAGE PROCESSING
disp('Steps 1 to 5 of RGB2Chl algorithm, described step by step in section 2.2 in Friedrichs et al. 2017')
% Step 1: Selection of sub image (region of interest)
[SampleData,ReferenceData]=regionofinterest(SampleImages,ReferenceImages,SubImageSize);

% Step 2: Red channel extraction of sub images
[SampleRedChannel,ReferenceRedChannel] = redchannelextract(SampleData,ReferenceData);

% Step 3: Illumination time extraction
[SampleIllumination,ReferenceIllumination] = illuminationtimeextract(SampleImages,ReferenceImages);
% Step 4:  Intensity calculation after Equation (2) in Friedrichs et al. 2017
[SampleIntensity,ReferenceIntensity] = intensitycalculation(SampleRedChannel,SampleIllumination,ReferenceRedChannel,ReferenceIllumination);

% Step 5: blank correction (base-line correction)
BaseLineCorrectedIntensity = blankcorrection(SampleIntensity,ReferenceIntensity);

% Averaging intensity of red channel along x and y axes (along pixel dimension) to get one intensity value per image
AveragedIntensityPerSubImage=squeeze(mean(mean(BaseLineCorrectedIntensity)));
disp('All processing steps done')
%% SUBFUNCTIONS FOR STEP 1 TO 5 %%
% Step 1:
function [SampleData,ReferenceData] = regionofinterest(SampleImages,ReferenceImages,SubImageSize)
    % Step 1: Selection of sub image (region of interest)
    disp('*** Step 1: Selection of sub image (region of intereset)')
    % allocate variables
    SampleData = uint8(zeros(2*SubImageSize,2*SubImageSize,3,length(SampleImages)));
    ReferenceData = uint8(zeros(2*SubImageSize,2*SubImageSize,3,length(ReferenceImages)));
    % for sample images
    for NoS = 1:length(SampleImages)
        % read data of image
        temp = imread([SampleImages(NoS).folder,'\',SampleImages(NoS).name]);
        % calculate center of image
        MP = size(temp)/2; 
        % define corner points for subsection
        CornerPoints=[MP(1)+SubImageSize-1,MP(2)+SubImageSize-1;...
                   MP(1)+SubImageSize-1,MP(2)-SubImageSize;...
                   MP(1)-SubImageSize,MP(2)+SubImageSize-11;...
                   MP(1)-SubImageSize,MP(2)-SubImageSize];
        % cut sub image
        SampleData(:,:,:,NoS) = temp(CornerPoints(3,1):CornerPoints(1,1),CornerPoints(2,2):CornerPoints(1,2),:);
        clear temp
    end
    % for reference images
    for NoB = 1:length(ReferenceImages)
        % read data of image
        temp = imread([ReferenceImages(NoB).folder,'\',ReferenceImages(NoB).name]);
        % calculate center of image
        MP = size(temp)/2; 
        % define corner points for subsection
        CornerPoints=[MP(1)+SubImageSize-1,MP(2)+SubImageSize-1;...
                   MP(1)+SubImageSize-1,MP(2)-SubImageSize;...
                   MP(1)-SubImageSize,MP(2)+SubImageSize-11;...
                   MP(1)-SubImageSize,MP(2)-SubImageSize];
        % cut sub image
        ReferenceData(:,:,:,NoB) = temp(CornerPoints(3,1):CornerPoints(1,1),CornerPoints(2,2):CornerPoints(1,2),:);
        clear temp
    end
end
% Step 2:
function [SampleRedChannel,ReferenceRedChannel] = redchannelextract(SampleData,ReferenceData)
    % Step 2: Red channel extraction of sub images    
    disp('*** Step 2: Red channel extraction')
    % allocate variables
    SampleRedChannel = double(zeros(size(SampleData,1),size(SampleData,2),size(SampleData,4)));
    ReferenceRedChannel = double(zeros(size(ReferenceData,1),size(ReferenceData,2),size(ReferenceData,4)));
    % for sample images
    for NoS = 1:size(SampleData,4)
        % red channel = channel number 1 in RGB images
        SampleRedChannel(:,:,NoS)=double(SampleData(:,:,1,NoS));
    end
    % for reference images
    for NoB = 1:size(ReferenceData,4)
        % red channel = channel number 1 in RGB images
        ReferenceRedChannel(:,:,NoB)=double(ReferenceData(:,:,1,NoB));
    end
end
% Step 3:
function [SampleIllumination,ReferenceIllumination] = illuminationtimeextract(SampleImages,ReferenceImages)
    % Step 3: Illumination time extraction    
    disp('*** Step 3: Illumination time extraction')
    % allocate variables
    SampleIllumination = zeros(length(SampleImages),1);
    ReferenceIllumination = zeros(length(ReferenceImages),1);
    % for sample images
    for NoS = 1:length(SampleImages)
        temp = imfinfo([SampleImages(NoS).folder,'\',SampleImages(NoS).name]);
        SampleIllumination(NoS,1)=temp.DigitalCamera.ExposureTime;
    end
    % for reference images
    for NoB = 1:length(ReferenceImages)
        temp = imfinfo([ReferenceImages(NoB).folder,'\',ReferenceImages(NoB).name]);
        ReferenceIllumination(NoB,1) = temp.DigitalCamera.ExposureTime;
    end
    
end
% Step 4:
function [SampleIntensity,ReferenceIntensity] = intensitycalculation(SampleRedChannel,SampleIllumination,ReferenceRedChannel,ReferenceIllumination)
    % Step 4: Intensity calculation
    disp('*** Step 4: Intensity calculation')
    % allocate variables
    SampleIntensity = zeros(size(SampleRedChannel));
    ReferenceIntensity = zeros(size(ReferenceRedChannel));
    % for sample images
    for NoS = 1:size(SampleRedChannel,3)
        SampleIntensity(:,:,NoS) = squeeze(SampleRedChannel(:,:,NoS))/SampleIllumination(NoS);
    end
    % for reference images
    for NoB = 1:size(ReferenceRedChannel,3)
        ReferenceIntensity(:,:,NoB) = squeeze(ReferenceRedChannel(:,:,NoB))/ReferenceIllumination(NoB);
    end
end
% Step 5: 
function BaseLineCorrectedIntensity = blankcorrection(SampleIntensity,ReferenceIntensity)
    % Step 5: Blank correction (base-line correction)
    disp('*** Step 5: Blank correction (base-line correction)')
    % allocate variables
    BaseLineCorrectedIntensity = zeros(size(SampleIntensity));
    % average reference intensity in case of more than one reference image
    ReferenceAve = mean(ReferenceIntensity,3);
    for NoS = 1:size(SampleIntensity,3)
        BaseLineCorrectedIntensity(:,:,NoS) = squeeze(SampleIntensity(:,:,NoS))-ReferenceAve;
    end
end